{--------------------------------------------------------------------------------
* Description : TscDataList
  Delphi collection component to store data (in design time)
  like SQL statements or values (Names and Values property of TStrings).
  The Find methods can be used the find items very quickly.
* Dates : March - November 2002
* Version : 1.1

* Author : Stefan Cruysberghs
* Email : stefancr@scip.be
* Website : http://www.scip.be
  Visit SCIP.be for other components, tools, articles, ...
--------------------------------------------------------------------------------
* $Archive: /Component Library/SC_Public/scDataList.pas $
* $Author: Stefancr $
* $Date: 19/06/03 15:59 $
* $Modtime: 17/11/02 18:08 $
* $Revision: 1 $
--------------------------------------------------------------------------------
* This component is free of charge.
* The author doesn't give a warranty for error free running
  of this component and he doesn't give any support.
* Suggestions and bugs can be send by email.
--------------------------------------------------------------------------------
Installation
* Install the TscDataList component by adding the unit scDataList.pas
  to a package. Remove the DCR file in the package. Compile and Install the package.
* The TscDataList component can be found in the tabsheet 'SC'
  of the component palette.
-------------------------------------------------------------------------------}

unit scDataList;

interface

uses Classes, SysUtils;

type
  // Just declare class of DataList because it is used in TscDataCollection
  TscDataList = class;

  // Collection Item
  // Create needs Collection
  TscDataItem = class(TCollectionItem)
  private
    FSLText1 : TStrings;
    FSLText2 : TStrings;
    FIntID : Integer;
    FStrDescription : String;
    procedure SetText1(const Value: TStrings);
    procedure SetText2(const Value: TStrings);
  protected
    // Display name of item in collection window
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    // ID
    property ID : Integer read FIntID write FIntID;
    // TStrings property 1 to store data (SQL)
    property Text1 : TStrings read FSLText1 write SetText1;
    // TStrings property 2 to store data (SQL)
    property Text2 : TStrings read FSLText2 write SetText2;
    // Description
    property Description : String read FStrDescription write FStrDescription;
  end;

  // Collection
  // Pointer to parent (DataList)
  // Items property with Get and Set to get Collection Item
  // In Create class of Collection Item is used
  TscDataCollection = class(TCollection)
  private
    FDataList : TscDataList;
    function GetItem(Index: Integer): TscDataItem;
    procedure SetItem(Index: Integer; const Value: TscDataItem);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(DataList : TscDataList);
    function GetOwner: TPersistent; override;

    function Add: TscDataItem;
    function AddItem(Item: TscDataItem; Index: Integer): TscDataItem;
    function Insert(Index: Integer): TscDataItem;

    // Find item index given the ID or the Description
    function FindItem(IntID : Integer) : Integer; overload;
    function FindItem(StrDescription : String) : Integer; overload;

    property Items[Index: Integer] : TscDataItem read GetItem write SetItem; default;
  end;

  // TComponent with a Collection property
  TscDataList = class(TComponent)
  private
    FDataCollection : TscDataCollection;
    procedure SetDataCollection(const Value: TscDataCollection);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Data : TscDataCollection read FDataCollection write SetDataCollection;
  end;

  procedure Register;

implementation

{$R scDataList.dcr}

procedure Register;
begin
  RegisterComponents('SC',[TscDataList]);
end;

//==============================================================================
// TscDataItem
//==============================================================================
constructor TscDataItem.Create(Collection: TCollection);
begin
  inherited;
  FSLText1 := TStringList.Create;
  FSLText2 := TStringList.Create;
end;

//------------------------------------------------------------------------------
destructor TscDataItem.Destroy;
begin
  FreeAndNil(FSLText1);
  FreeAndNil(FSLText2);  
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscDataItem.SetText1(const Value: TStrings);
begin
  FSLText1.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscDataItem.SetText2(const Value: TStrings);
begin
  FSLText2.Assign(Value);
end;

//------------------------------------------------------------------------------
function TscDataItem.GetDisplayName: string;
begin
  if Trim(FStrDescription) <> '' then
    Result := 'ID = '+IntToStr(FIntID)+'  Descr = '+FStrDescription
  else
    Result := 'ID = '+IntToStr(FIntID);
end;

//==============================================================================
// TscDataCollection
//==============================================================================

//------------------------------------------------------------------------------
constructor TscDataCollection.Create(DataList : TscDataList);
begin
  if DataList <> nil then
    inherited Create(TscDataItem);
  FDataList := DataList;
end;

//------------------------------------------------------------------------------
function TscDataCollection.GetOwner: TPersistent;
begin
  Result:=FDataList;
end;

//------------------------------------------------------------------------------
function TscDataCollection.GetItem(Index: Integer): TscDataItem;
begin
  result := TscDataItem(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
procedure TscDataCollection.SetItem(Index: Integer; const Value: TscDataItem);
begin
  inherited SetItem(Index, Value);
end;

//------------------------------------------------------------------------------
function TscDataCollection.Add: TscDataItem;
begin
  Result := TscDataItem(inherited Add);
end;

//------------------------------------------------------------------------------
function TscDataCollection.AddItem(Item: TscDataItem; Index: Integer): TscDataItem;
begin
  if Item = nil then
    Result := TscDataItem.Create(Self)
  else
  begin
    Result := Item;
    if Assigned(Item) then
    begin
      Result.Collection := Self;
      if Index < 0 then
        Index := Count - 1;
      Result.Index := Index;
    end;
  end;
end;

//------------------------------------------------------------------------------
function TscDataCollection.Insert(Index: Integer): TscDataItem;
begin
  Result := AddItem(nil, Index);
end;

//------------------------------------------------------------------------------
procedure TscDataCollection.Update(Item: TCollectionItem);
begin
  //if Assigned(Item) then
end;

//------------------------------------------------------------------------------
function TscDataCollection.FindItem(IntID: Integer): Integer;
var
  i : Integer;
begin
  Result := -1;
  i:=0;
  repeat
    if Items[i].FIntID = IntID then
      Result := i;
    Inc(i);
  until (Result <> -1) or (Count = i);
end;

//------------------------------------------------------------------------------
function TscDataCollection.FindItem(StrDescription: String): Integer;
var
  i : Integer;
begin
  Result := -1;
  i:=0;
  repeat
    if CompareText(Items[i].FStrDescription,StrDescription) = 0 then
      Result := i;
    Inc(i);
  until (Result <> -1) or (Count = i);
end;

//==============================================================================
// TscDataList
//==============================================================================
constructor TscDataList.Create(AOwner: TComponent);
begin
  inherited;
  FDataCollection := TscDataCollection.Create(Self);
end;

//------------------------------------------------------------------------------
destructor TscDataList.Destroy;
begin
  FDataCollection.Clear;
  FreeAndNil(FDataCollection);
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscDataList.SetDataCollection(const Value: TscDataCollection);
begin
  FDataCollection := Value;
end;

end.
